@@ -125,7 +125,7 @@ main (int argc, char * argv[])
 				continue;
 
 			} else if (info->type == GNOME_VFS_FILE_TYPE_REGULAR
-				   && is_ext (desktops[i], ".directory")
+				   && g_str_has_suffix (desktops [i], ".directory")
 				   && !create_new) {
 				char *dirname = g_path_get_dirname (uri);
 				dlg = panel_edit_direntry (dirname, NULL,
@@ -133,7 +133,7 @@ main (int argc, char * argv[])
 				g_free (dirname);
 			
 			} else if (info->type == GNOME_VFS_FILE_TYPE_REGULAR
-				   && is_ext (desktops[i], ".desktop")
+				   && g_str_has_suffix (desktops [i], ".desktop")
 				   && !create_new) {
 				char *dirname = g_path_get_dirname (uri);
 				dlg = panel_edit_dentry (uri, dirname,
@@ -146,7 +146,7 @@ main (int argc, char * argv[])
 					 uri);
 			}
 
-		} else if (is_ext (desktops[i], ".directory")
+		} else if (g_str_has_suffix (desktops [i], ".directory")
 			   && !create_new) {
 			/* a non-existant file.  Well we can still edit that sort
 			 * of.  We will just create it new */
@@ -155,7 +155,7 @@ main (int argc, char * argv[])
 						   gdk_screen_get_default ());
 			g_free (dirname);
 		
-		} else if (is_ext (desktops[i], ".desktop")
+		} else if (g_str_has_suffix (desktops [i], ".desktop")
 			   && !create_new) {
 			/* a non-existant file.  Well we can still edit that sort
 			 * of.  We will just create it new */
