@@ -568,7 +568,8 @@ panel_profile_get_toplevel_orientation (PanelToplevel *toplevel)
 	key = panel_profile_get_toplevel_key (toplevel, "orientation");
 	str = gconf_client_get_string (client, key, NULL);
 
-	panel_profile_map_orientation_string (str, &orientation);
+	if (!panel_profile_map_orientation_string (str, &orientation))
+	    orientation = panel_toplevel_get_orientation (toplevel);
 
 	g_free (str);
 
@@ -805,7 +806,7 @@ panel_profile_queue_toplevel_location_change (PanelToplevel          *toplevel,
 			panel_profile_get_toplevel_key (toplevel, "y_centered"),
 			change->y_centered);
 
-	if (change->orientation)
+	if (change->orientation_changed)
 		gconf_change_set_set_string (
 			queued_changes,
 			panel_profile_get_toplevel_key (toplevel, "orientation"),
@@ -1521,6 +1522,7 @@ panel_profile_load_object (GConfClient       *client,
 	char            *toplevel_id;
 	int              position;
 	gboolean         right_stick;
+	gboolean         locked;
 
 	key = panel_gconf_full_key (type, current_profile, id, "object_type");
 	type_string = gconf_client_get_string (client, key, NULL);
@@ -1541,7 +1543,15 @@ panel_profile_load_object (GConfClient       *client,
 	key = panel_gconf_full_key (type, current_profile, id, "panel_right_stick");
 	right_stick = gconf_client_get_bool (client, key, NULL);
 
-	panel_applet_queue_applet_to_load (id, object_type, toplevel_id, position, right_stick);
+	key = panel_gconf_full_key (type, current_profile, id, "locked");
+	locked = gconf_client_get_bool (client, key, NULL);
+
+	panel_applet_queue_applet_to_load (id,
+					   object_type,
+					   toplevel_id,
+					   position,
+					   right_stick,
+					   locked);
 }
 
 static void
@@ -1580,6 +1590,10 @@ panel_profile_delete_dir (GConfClient       *client,
 
 	key = panel_gconf_sprintf (PANEL_CONFIG_DIR "/%s/%s/%s",
 				   current_profile, type_str, id);
+
+	if (type == PANEL_GCONF_TOPLEVELS)
+	    gconf_client_remove_dir (client, key, NULL);
+
 	panel_gconf_clean_dir (client, key);
 }
 
